#include "ax.h"

void axEvent::callBack(XEvent &event)
{
	if(event.type == event_)
		(parent_->*callBack_)(event);
}
	
axEvent::axEvent(axWindow *parent, int event, EventCallBack callBack)
	: parent_(parent), event_(event), callBack_(callBack)
{
	// mask2event mapping from Xlib C Library doc, page 187.
	switch(event_){
		case KeyPress:
			mask_ = KeyPressMask; break;
		case KeyRelease:
			mask_ = KeyReleaseMask; break;
		case ButtonPress:
			mask_ = ButtonPressMask; break;
		case ButtonRelease:
			mask_ = ButtonReleaseMask; break;
		case MotionNotify:
			mask_ = Button1MotionMask | Button2MotionMask | Button3MotionMask |
							Button4MotionMask | Button5MotionMask | ButtonMotionMask |
							PointerMotionMask; break;
		case EnterNotify:
			mask_ = EnterWindowMask; break;
		case LeaveNotify:
			mask_ = LeaveWindowMask; break;
		case FocusIn:
		case FocusOut:
			mask_ = FocusChangeMask; break;
		case KeymapNotify:
			mask_ = KeymapStateMask; break;
		case Expose:
		case GraphicsExpose:
		case NoExpose:
			mask_ = ExposureMask; break;
		case ResizeRequest:
			mask_ = ResizeRedirectMask; break;
		case PropertyNotify:
			mask_ = PropertyChangeMask; break;
		case ColormapNotify:
			mask_ = ColormapChangeMask; break;
		case CirculateNotify:
		case ConfigureNotify:
		case DestroyNotify:
		case GravityNotify:
		case MapNotify:
		case UnmapNotify:
		case ReparentNotify:
			mask_ = StructureNotifyMask | SubstructureNotifyMask; break;
		case CreateNotify:
			mask_ = SubstructureNotifyMask; break;
		case MapRequest:
		case ConfigureRequest:
		case CirculateRequest:
			mask_ = SubstructureRedirectMask; break;
		case VisibilityNotify:
			mask_ = VisibilityChangeMask; break;
		case ClientMessage:
		case MappingNotify:
		case SelectionClear:
		case SelectionRequest:
		case SelectionNotify:
			// XXX is this a problem???
			log("axEvent::axEvent(): unmaskable event requested");
			mask_ = NoEventMask; break; // these cannot be masked

		// these masks do not have associated events
		//  PointerMotionHintMask
 		//  OwnerGrabButtonMask

		default:
			log("axEvent::axEvent(): unknown event type");
			mask_ = NoEventMask; break;
	}
}

/* original xosview event map
		case ButtonPress: mask_ = ButtonPressMask; break;
		case ButtonRelease: mask_ = ButtonReleaseMask; break;
		case EnterNotify: mask_ = EnterWindowMask; break;
		case LeaveNotify: mask_ = LeaveWindowMask; break;
		case MotionNotify: mask_ = PointerMotionMask; break;
		case FocusIn:
		case FocusOut: mask_ = FocusChangeMask; break;
		case KeymapNotify: mask_ = KeymapStateMask; break;
		case KeyPress: mask_ = KeyPressMask; break;
		case KeyRelease: mask_ = KeyReleaseMask; break;
		case MapNotify:
		case SelectionClear:
		case SelectionNotify:
		case SelectionRequest:
		case ClientMessage:
		case MappingNotify: mask_ = NoEventMask; break;
		case Expose:
		case GraphicsExpose:
		case NoExpose: mask_ = ExposureMask; break;
		case ColormapNotify: mask_ = ColormapChangeMask; break;
		case PropertyNotify: mask_ = PropertyChangeMask; break;
		case UnmapNotify:
		case ReparentNotify:
		case GravityNotify:
		case DestroyNotify:
		case CirculateNotify:
		case ConfigureNotify: mask_ = StructureNotifyMask | SubstructureNotifyMask;
			break;
		case CreateNotify: mask_ = SubstructureNotifyMask; break;
		case VisibilityNotify: mask_ = VisibilityChangeMask; break;
		// The following are used by window managers
		case CirculateRequest:
		case ConfigureRequest:
		case MapRequest: mask_ = SubstructureRedirectMask; break;
		case ResizeRequest: mask_ = ResizeRedirectMask; break;
		default:
			cerr << "axEvent::Event(): unknown event type: " << event_ << endl;
			mask_ = NoEventMask;
			break;
*/
