#ifndef _AXVECTOR_H_
#define _AXVECTOR_H_

#include <iostream.h>
#include "axexception.h"

template <class T>
class axVector{
	// contains by reference (pointer)
	// IDEA make a linked list of N blocks of `increment' pointers, and add
	// blocks of pointers when needed; upside: no copies, downside: extra list
	// handling. Funny: this would be a Vector of Vectors  :)
	private:
	int capacity_;												// initial capacity
	int increment_;												// capacity added when full
	int index_;														// -1 == empty
	T **objects_;													// ptr to array of T ptrs

	void checkBounds(){
		if(index_ == capacity_){
			int newCapacity = capacity_ + increment_;
			//printf("enlarging vector from %d to %d\n", capacity_, newCapacity);
			T **tmp = new (T*)[newCapacity];
			for(int i = 0; i < capacity_; i++)
				tmp[i] = objects_[i]; // copy the pointers
			T **oldObjects = objects_;
			objects_ = tmp;
			delete[] oldObjects;
			capacity_ = newCapacity;
		}
	}

	public:
	axVector(uint capacity = 100, uint increment = 100)
		: capacity_(capacity), increment_(increment), index_(-1),
			objects_(new (T*)[capacity_]) { }

	~axVector(){
		for(int i = 0; i <= index_; i++)
			delete objects_[i];
		delete[] objects_;
	}

	int size() const { return index_ + 1; }

	void add(T *obj){
		checkBounds();
		objects_[++index_] = obj;
	}

	T& operator[](int i) const {
		if(i >= 0 && i <= index_)
			return *objects_[i];
		else
			//throw axException("axVector::operator[]: index out of range");
			cerr << "axVector::operator[]: index out of range" << endl;
			throw;
	}

	// iterator stuff, example:
	// Vector<T> x; for(T *t = x.begin(); !x.end(); t = x.next()) { }
	//T *begin() { iterator_ = 0; return objects_[iterator_]; }
	//bool end() { return iterator_ == size_; }
	//T *next() { return objects_[++iterator_]; }
};

#endif _AXVECTOR_H_
