#include "ax.h"

class Shape;
class Model;
class Rectangle;
class Vertex;
class Segment;
class Draw;

class Shape{
	public:
	virtual void paint(Draw*) = 0;
	// having a virtual destructor here enables
	// Vector to call destructors on all shapes
	virtual ~Shape() { }
};

class Draw : public axWindow{
	private:
	axVector<Shape> shapes_;

	public:
	Draw() : axWindow("Draw", 300, 350) { }
	~Draw() { }
	void populate();
	void onExpose(XEvent &event){
		for(int i = 0; i < shapes_.size(); i++)
			shapes_[i].paint(this);
	}
};

class Vertex : public Shape{
	private:
	int x_, y_;

	public:
	Vertex(int x = 0, int y = 0) : x_(x), y_(y) { }
	void paint(Draw *d){
		d->drawRectangle(x_ - 1, y_ - 1, 3, 3);
	}
};

class Rectangle : public Shape{
	private:
	int x_, y_, width_, height_;

	public:
	Rectangle(int x, int y, int width, int height)
		: x_(x), y_(y), width_(width), height_(height) { }
	void paint(Draw *d){
		d->drawRectangle(x_, y_, width_, height_);
	}
};

class Segment : public Shape{
	private:
	int x1_, y1_, x2_, y2_;

	public:
	Segment(int x1, int y1, int x2, int y2)
		: x1_(x1), y1_(y1), x2_(x2), y2_(y2) { }
	void paint(Draw *d){
		d->drawLine(x1_, y1_, x2_, y2_);
	}
};

void Draw::populate(){
	// put here to avoid mutual dependencies with shapes
	for(int i = 0; i < 100; i += 5){
		shapes_.add(new Segment(i, 20, i, 120));
		shapes_.add(new Vertex(i + 130, i + 20));
		shapes_.add(new Rectangle(i, i + 130, i, i));
	}
}

int main()
{
	Draw d;
	d.trace(true);
	d.populate();
	d.run();
	return 0;
}

/* nice matrix interface example
 * from http://www.cerfnet.com/~mpcline/c++-faq-lite/operator-overloading.html
class Matrix {
public:
	Matrix(unsigned rows, unsigned cols);
	double& operator() (unsigned row, unsigned col);
	double  operator() (unsigned row, unsigned col) const;
	// ...
 ~Matrix();                              // Destructor
	Matrix(const Matrix& m);               // Copy constructor
	Matrix& operator= (const Matrix& m);   // Assignment operator
	// ...
private:
	unsigned rows_, cols_;
	double* data_;
};

inline
Matrix::Matrix(unsigned rows, unsigned cols)
	: rows_ (rows),
		cols_ (cols),
		data_ (new double[rows * cols])
{
	if (rows == 0 || cols == 0)
		throw BadIndex("Matrix constructor has 0 size");
}

inline
Matrix::~Matrix()
{
	delete[] data_;
}

inline
double& Matrix::operator() (unsigned row, unsigned col)
{
	if (row >= rows_ || col >= cols_)
		throw BadIndex("Matrix subscript out of bounds");
	return data_[cols_*row + col];
}

inline
double Matrix::operator() (unsigned row, unsigned col) const
{
	if (row >= rows_ || col >= cols_)
		throw BadIndex("const Matrix subscript out of bounds");
	return data_[cols_*row + col];
} 

Then you can access an element of Matrix m using m(i,j) rather than m[i][j]: 

int main()
{
	Matrix m(10,10);
	m(5,8) = 106.15;
	cout << m(5,8);
	// ...
} 
*/
