#include "matrix.h"

// XXX make all members take h into account

// init consts, references, and members
Vector::Vector(double x, double y, double z, double h)
	:	x_(data_[0]), y_(data_[1]), z_(data_[2]), h_(data_[3]),
		X(0), Y(1), Z(2), H(3), ROWS(4)
{
	x_ = x; y_ = y; z_ = z; h_ = h;
}

/*
Vector::Vector(const Vector& v)
{
	// copy constructor
	// XXX we have to reinit al refs and consts...
	x_ = v.x();
	y_ = v.y();
	z_ = v.z();
	h_ = v.h();
}
*/

Vector::~Vector()
{
}

double& Vector::operator()(uint row)
{
	// assignments to elements: m(i,j) = x;
	if(row >= ROWS)
		throw MatrixException("Vector::operator(): subscript out of bounds");
	return data_[row];
}

double Vector::operator()(uint row) const
{
	// constant use of elements, e.g. in operators with const Vector& params
	if (row >= ROWS)
		throw MatrixException("Vector::operator() const: subscript out of bounds");
	return data_[row];
}

Vector& Vector::operator=(const Vector& m)
{
	// override of default `bit-wise copy' assignment: neccessary due to data_
	// member which needs indirection to be copied
	/*
	if(ROWS != m.ROWS)
		throw MatrixException("Vector::operator=: vectors have different sizes");
	memcpy(data_, m.data_, sizeof(double) * ROWS);
	*/
	// XXX casting away to shut up gcc-2.95.2, maximum hair
	data_ = ((Vector)m).data_;
	return *this;
}

Vector Vector::operator+() const
{
	return *this;
}

Vector Vector::operator-() const
{
	return Vector(-x_, -y_, -z_);
}

Vector Vector::operator+(const Vector& v)
{
	return Vector(x_ + v.x(), y_ + v.y(), z_ + v.z());
}

Vector Vector::operator-(const Vector& v)
{
	return Vector(x_ - v.x(), y_ - v.y(), z_ - v.z());
}

Vector Vector::operator*(const Vector& v)
{
	return Vector(x_ * v.x(), y_ * v.y(), z_ * v.z());
}

Vector Vector::operator*(double d)
{
	return Vector(x_ * d, y_ * d, z_ * d);
}

Vector Vector::operator/(const Vector& v)
{
	return Vector(x_ / v.x(), y_ / v.y(), z_ / v.z());
}

Vector Vector::operator/(double d)
{
	return Vector(x_ / d, y_ / d, z_ / d);
}

Vector Vector::operator==(const Vector& v)
{
	return (x_ == v.x()) && (y_ == v.y()) && (z_ == v.z());
}

double Vector::magnitude()
{
	return sqrt(x_ * x_ + y_ * y_ + z_ * z_);
}

Vector Vector::normalize()
{
	return *this / magnitude();
}

double Vector::dotProduct(const Vector& v)
{
	return x_ * v.x() + y_ * v.y() + z_ * v.z();
}

Vector Vector::crossProduct(const Vector& v)
{
	return Vector(
		y_ * v.z() - z_ * v.y(),
		z_ * v.x() - x_ * v.z(),
		x_ * v.y() - y_ * v.x()
	);
}

ostream& operator<<(ostream& s, const Vector& v)
{
	s << "(";
	for(uint r = 0; r < v.ROWS; r++){
		if(r > 0)
			s << ", ";
		s << setprecision(3) << v(r);
		//s << setprecision(2) << setw(5) << v(r);
	}
	return s << ")\n";
	/*
	s << "v--\n";
	for(uint r = 0; r < v.rows(); r++)
		s << "| " << v(r) << "\n";
	return s;
	*/
}
