#include <time.h>
#include "x3d.h"

Model::Model(char *modelfile) : axWindowDBE("x3d", 512, 512)
{
	zviewplane_ = 50;
	zprojpoint_ = 200;
	hidden_ = 0;
	color_ = 0;
	xangle = 0, yangle = 0, zangle = 0; // current rotation status
	xstep  = 0, ystep  = 0, zstep  = 0; // inc/dec control
	mousectrl = 0;											// control object with mouse
	modelfile_ = modelfile;
}	

void Model::run()
{
	struct timeval t1, t2;
	double t1f, t2f, tdiff;
	char msg[0x100];

	//trace(true);
	// NOTE always addEvent before calling init
	addEvent(new axEvent((axWindow*)this, MotionNotify,
		(EventCallBack)&Model::onMotionNotify));
	init();

	object = new Object(this);
	if(modelfile_){
		if(!object->readFromFile(modelfile_))
			panic("Model::run(): error reading model file\n");
	}
	else
		object->makeCube(Vector(0, 0, 0), 40);
	object->makeAllVisible();
	/*Object *o2=new Object(this);
	o2->readFromFile("models/cube.3d");
	o2->makeAllVisible();
	o2->location(150, 150, 0);*/
	light_ = new Vector(100, 100, 100);

	while(!done()){
		checkEvents();

		gettimeofday(&t1, NULL);

		xangle = (xangle + xstep) % 360;
		yangle = (yangle + ystep) % 360;
		zangle = (zangle + zstep) % 360;
		Matrix m = Matrix::rotateXYZMatrix(Vector(xangle, yangle, zangle));

		//clearWindow();
		object->draw(m);
		//o2->draw(m);

		gettimeofday(&t2, NULL);
		t1f = t1.tv_sec + t1.tv_usec / 1000000.0;
		t2f = t2.tv_sec + t2.tv_usec / 1000000.0;
		tdiff = t2f - t1f;

		setForeground(0x000000);
		sprintf(msg, "%7.3f fps | xa:%4d ya:%4d za:%4d | xs:%3d ys:%3d zs:%3d",
			1.0 / tdiff, xangle, yangle, zangle, xstep, ystep, zstep);
		drawString(4, 16, msg);
		sprintf(msg, "zprojpoint:%4d zviewplane:%4d", zprojpoint_, zviewplane_);
		drawString(4, 30, msg);
		swapBuffers();

		usleep(20*1000); // in usec
	}
}

void Model::onExpose(XEvent &ev)
{
	XWindowAttributes wa;
	if(ev.xexpose.count == 0){
		getWindowAttributes(&wa);
		width(wa.width);
		height(wa.height);
	}
}

void Model::onKeyPress(XEvent &ev)
{
	KeySym ksym;
	// AX put this translation in superclass and pass KeySym in
	ksym = XLookupKeysym((XKeyEvent*)&ev, 0);
	switch(ksym){ // see X11/keysymdef.h for keys
		case XK_q:
			done(1);
			break;

		case XK_h:
			hidden_ ^= 1; // flip
			object->makeAllVisible();
			if(!hidden_)
				color_ = 0;
			break;

		case XK_c:
			color_ ^= 1;
			hidden_ = 1;
			break;

		/* old keys
		case XK_KP_Left: ystep--; break;		// 4/6 for y-axis
		case XK_KP_Right: ystep++; break;
		case XK_KP_Down: xstep++; break;		// 2/8 for x-axis
		case XK_KP_Up: xstep--; break;			// reversed for visual feeling
		case XK_KP_End: zstep++; break;			// 1/9 for z-axis
		case XK_KP_Page_Up: zstep--; break;
		case XK_KP_Begin:										// stop rotation (+restore angles)
		*/
		case XK_j: ystep--; break;					// j/k for y-axis
		case XK_k: ystep++; break;
		case XK_m: xstep++; break;					// m/i for x-axis
		case XK_i: xstep--; break;					// reversed for visual feeling
		case XK_n: zstep++; break;					// n/o for z-axis
		case XK_o: zstep--; break;
		case XK_p:													// stop rotation (+restore angles)
			if(!(xstep || ystep || zstep))
				xangle = yangle = zangle = 0;
			xstep = ystep = zstep = 0;
			break;
		case XK_a: zprojpoint_ -= 10; break;	// move closer
		case XK_z: zprojpoint_ += 10; break;	// move away
		case XK_s: zviewplane_ -= 10; break;	// move closer
		case XK_x: zviewplane_ += 10; break;	// move away
	}
}

void Model::onButtonPress(XEvent &ev)
{
	lastx = ev.xbutton.x;
	lasty = ev.xbutton.y;
	mousectrl = 1;
}

void Model::onButtonRelease(XEvent &ev)
{
	mousectrl = 0;
	(void)ev;
}

void Model::onMotionNotify(XEvent &ev)
{
	if(mousectrl){
		xangle = (xangle + (ev.xmotion.y - lasty)) % 360;
		yangle = (yangle + (ev.xmotion.x - lastx)) % 360;
		lastx = ev.xmotion.x;
		lasty = ev.xmotion.y;
	}
}
