#include <math.h>
#include "x3d.h"

Plane::Plane(Object *object)
{
	next_ = NULL;
	object_ = object;
}

void Plane::removeHidden(Vector *view)
{
	Vector v1 = view[vertices_[0]] - view[vertices_[1]]; // vector from v1 to v0
	Vector v2 = view[vertices_[1]] - view[vertices_[2]]; // vector from v2 to v1
	normal_ = v1.crossProduct(v2);
	normal_ = normal_.normalize();

	// simple viewing system allows simplified normal testing
	visible_ = (normal_.z() > 0) ? 1 : 0;

	// full calculation (Hearn, pp. 471):
	// Vector viewdir(0, 0, -1);
	// visible_ = (viewdir.dotProduct(normal_) > 0) ? 0 : 1;
}

void Plane::setColor(Vector &light)
{
	double angle; // angle between plane normal and light normal
	double normmag, lightmag, dotprod;

	if(visible_){
		normmag  = normal_.magnitude();
		lightmag = light.magnitude();
		dotprod  = normal_.dotProduct(light);
		angle    = acos(dotprod / (normmag * lightmag)) / M_PI; // in degrees
		color_   = 0x40FF40 - ((int)(0xC0 * angle) << 8);
		//pp->color = NROFCOLORS - ceil((double)(NROFCOLORS * angle) / 180.0);
		//pp->Color = NROFCOLORS - ceil(double(NROFCOLORS * angle) / 180.0);
		//pp->Color = NROFCOLORS - (double(NROFCOLORS * angle) / 180.0);
	}
}
	
void Plane::draw(Model *model, Vector *view, int coloring)
{
	const int PBUFFER = 10;
	XPoint pts[PBUFFER];
	int xcenter = object_->model()->width() / 2;
	int ycenter = object_->model()->height() / 2;
	Vector *loc = object_->location();
	double xloc = loc->x();
	double yloc = loc->y();
	const int VF = 4;

	if(visible_){
		if(nvertices_ > PBUFFER){
			fprintf(stderr, "Plane::draw(): nvertices > %d", PBUFFER);
			exit(1);
		}
		for(int i = 0; i <= nvertices_; i++){
			// FIXME move VF and x/ycenter stuff to Object::trans3Dto2D
			// better still: get rid of VF alltogether.
			// FIXME errm, doesn't "ycenter - ..." invert the coord syst?
			// FIXME {x,y}loc must be tran3dto2d'd
			//pts[i].x = short(VF * view[vertices_[i % nvertices_]].x + xcenter);
			//pts[i].y = short(ycenter - VF * view[vertices_[i % nvertices_]].y);
			pts[i].x = short(VF * view[vertices_[i % nvertices_]].x() +
				xcenter + xloc);
			pts[i].y = short(ycenter - yloc - VF *
				view[vertices_[i % nvertices_]].y());
		}
		//pts[4] = pts[0];
		if(coloring){
			model->setForeground(color_);
			model->fillPolygon(pts, nvertices_, Convex, CoordModeOrigin);
			//XSetForeground(dpy, gc, 0x000000);
			//XDrawLines(dpy, w, gc, pts, pp->nvertices, CoordModeOrigin);
		}
		else{
			model->setForeground(0);
			model->drawLines(pts, nvertices_ + 1, CoordModeOrigin);
		}
	}
}
